#include "esp_bt_main.h"
#include "esp_bt_device.h"
#include"esp_gap_bt_api.h"
#include "esp_err.h"

#include <Bounce2.h>
#include <BleKeyboard.h>
BleKeyboard bleKeyboard;//("BT-Media", "ESP32-duke24", 74);
int volume_down = 19;
int next_track = 18;
int previous_track = 5;
int play_pause = 4;
int volume_up = 15;

#define REMOVE_BONDED_DEVICES 0   // <- Set to 0 to view all bonded devices addresses, set to 1 to remove

#define PAIR_MAX_DEVICES 9
uint8_t pairedDeviceBtAddr[PAIR_MAX_DEVICES][6];
char bda_str[18];


#define NUM_BUTTONS 5
const uint8_t BUTTON_PINS[NUM_BUTTONS] = {play_pause, previous_track, volume_up, next_track, volume_down};

Bounce * buttons = new Bounce[NUM_BUTTONS];
unsigned long buttonPressTimeStampu;
unsigned long buttonPressTimeStampd;
void setup() {
  Serial.begin(115200);
  Serial.println("Starting BLE work!");
  checkReset() ;
  bleKeyboard.begin();
  for (int i = 0; i < NUM_BUTTONS; i++) {
    buttons[i].attach( BUTTON_PINS[i] , INPUT_PULLUP  );       //setup the bounce instance for the current button
    buttons[i].interval(100);              // interval in ms
  }
}

void loop() {
  for (int i = 0; i < NUM_BUTTONS; i++)  {
    // Update the Bounce instance :
    buttons[i].update();
    // If it fell, flag the need to toggle the LED
  }
  if ( buttons[0].fell() ) {
    Serial.println("button Pushed");
    if (bleKeyboard.isConnected()) {
      Serial.println("Pressing the Play/Pause.Button via the Bluetooth keyboard");
      bleKeyboard.write(KEY_MEDIA_PLAY_PAUSE);
      bleKeyboard.write(L);
    }
  }

  if (  buttons[3].fell() ) {
    Serial.println("next button Pushed");
    if (bleKeyboard.isConnected()) {
      Serial.println("Pressing the next-button via the Bluetooth keyboard");
      bleKeyboard.write(KEY_MEDIA_NEXT_TRACK);
      bleKeyboard.write(KEY_UP_ARROW);
    }
  }

  if(digitalRead(volume_up)== LOW){
      Serial.println("VOLUME UP");
      bleKeyboard.write(KEY_MEDIA_VOLUME_UP);
      bleKeyboard.write(+);
      delay(500);
    }

    else if(digitalRead(volume_down)== LOW){
      Serial.println("VOLUME DOWN");
      bleKeyboard.write(KEY_MEDIA_VOLUME_DOWN);
      bleKeyboard.write(-);
      delay(500);
    }
    
  if ( buttons[1].fell() ) {
    Serial.println("prev button Pushed");
    if (bleKeyboard.isConnected()) {
      Serial.println("Pressing the next-button via the Bluetooth keyboard");
      bleKeyboard.write(KEY_MEDIA_PREVIOUS_TRACK);
      bleKeyboard.write(KEY_DOWN_ARROW);
    }
  }

  if ( buttons[2].fell() ) {
    Serial.println("up volume button pressed");
    if (bleKeyboard.isConnected()) {
      if ( millis() - buttonPressTimeStampu <= 1000) {
        Serial.println("Volume up");
        bleKeyboard.write(KEY_MEDIA_VOLUME_UP);
        Serial.println( millis() - buttonPressTimeStampu );
        buttonPressTimeStampu = millis();
      } else {
        Serial.println("Volume up 10x");
        for (int i = 0; i < 1; i++) {
          bleKeyboard.write(KEY_MEDIA_VOLUME_UP);
          delay(100);
        }
        Serial.println( millis() - buttonPressTimeStampu );
        buttonPressTimeStampu = millis();
      }
    }
  }
  if (buttons[4].fell() ) {
    Serial.println("volume reducing");
    if (bleKeyboard.isConnected()) {
      if ( millis() - buttonPressTimeStampd <= 1000) {
        Serial.println("Volume down");
        bleKeyboard.write(KEY_MEDIA_VOLUME_DOWN);
        Serial.println( millis() - buttonPressTimeStampd );
        buttonPressTimeStampd = millis();
      } else {
        Serial.println("Volume down 10x");
        for (int i = 0; i < 1; i++) {
          bleKeyboard.write(KEY_MEDIA_VOLUME_DOWN);
          delay(100);
        }
        Serial.println( millis() - buttonPressTimeStampd );
        buttonPressTimeStampd = millis();
      }
    }
  }
}
bool initBluetooth() {
  if (!btStart()) {
    Serial.println("Failed to initialize controller");
    return false;
  }

  if (esp_bluedroid_init() != ESP_OK) {
    Serial.println("Failed to initialize bluedroid");
    return false;
  }

  if (esp_bluedroid_enable() != ESP_OK) {
    Serial.println("Failed to enable bluedroid");
    return false;
  }
  return true;
}

char *bda2str(const uint8_t* bda, char *str, size_t size) {
  if (bda == NULL || str == NULL || size < 18) {
    return NULL;
  }
  sprintf(str, "%02x:%02x:%02x:%02x:%02x:%02x",
          bda[0], bda[1], bda[2], bda[3], bda[4], bda[5]);
  return str;
}
void checkReset() {
  //initBluetooth();
  Serial.print("ESP32 bluetooth address: "); Serial.println(bda2str(esp_bt_dev_get_address(), bda_str, 18));
  // Get the numbers of bonded/paired devices in the BT module
  int count = esp_bt_gap_get_bond_device_num();
  if (!count) {
    Serial.println("No bonded device found.");
  } else {
    Serial.print("Bonded device count: "); Serial.println(count);
    if ( abs(count) > PAIR_MAX_DEVICES) {
      //count = PAIR_MAX_DEVICES;
      Serial.print("Reset bonded device count: "); Serial.println(count);
      REMOVE_BONDED_DEVICES == 1;
    }
    esp_err_t tError =  esp_bt_gap_get_bond_device_list(&count, pairedDeviceBtAddr);
    if (ESP_OK == tError) {
      for (int i = 0; i < count; i++) {
        Serial.print("Found bonded device # "); Serial.print(i); Serial.print(" -> ");
        Serial.println(bda2str(pairedDeviceBtAddr[i], bda_str, 18));
        if (REMOVE_BONDED_DEVICES) {
          esp_err_t tError = esp_bt_gap_remove_bond_device(pairedDeviceBtAddr[i]);
          if (ESP_OK == tError) {
            Serial.print("Removed bonded device # ");
          } else {
            Serial.print("Failed to remove bonded device # ");
          }
          Serial.println(i);
        }
      }
    }
  }
}
